--------------------------------------------------------------------------------
-- Copyright (c) 2000 by Trenz Electronic.
-- Duenner Kirchweg 77, 32257 Buende, Germany, www.trenz-electronic.de
--     
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--     
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--     
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
--------------------------------------------------------------------------------
-- Project:      Application Note: SoC with Xilinx Virtex
-- File:         xsvGlue.vhd
-- Description:  USB Macro glue logic.
-- Version:      FB, 2001jan29
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity xsvGlue is
	port(
		clk:           in  STD_LOGIC;                     -- 24MHz uC clock
		rst:           in  STD_LOGIC;                     -- global reset
		-- USB EP0 interface
		usb_stat:      in  STD_LOGIC_VECTOR(7 downto 0);  -- EP0 status
		usb_ctrl:      out STD_LOGIC_VECTOR(7 downto 0);  -- EP0 control
		usb_wrctrl:    out STD_LOGIC;                     -- ... write enable
		usb_adx:       out STD_LOGIC_VECTOR(2 downto 0);  -- EP0 FIFO address
		usb_rdd:       in  STD_LOGIC_VECTOR(7 downto 0);  -- EP0 FIFO read
		usb_wrd:       out STD_LOGIC_VECTOR(7 downto 0);  -- EP0 FIFO write
		usb_wren:      out STD_LOGIC;                     -- ... write enable
		usb_dadx:      out STD_LOGIC_VECTOR(6 downto 0);  -- device address
		usb_wradx:     out STD_LOGIC;                     -- ... write enable
		-- USB EP1 interface
		usb_txd1:      out STD_LOGIC_VECTOR(7 downto 0);  -- EP1 data
		usb_txen:      in  STD_LOGIC;
		usb_in_trac1:  in  STD_LOGIC;
		usb_togglein1: out STD_LOGIC;
		usb_datain1:   out STD_LOGIC;
		-- TE51 microcontroller
		uc_addr:       in  STD_LOGIC_VECTOR(15 downto 0); -- address bus
		uc_rdd:        out STD_LOGIC_VECTOR(7 downto 0);  -- data read
		uc_wrd:        in  STD_LOGIC_VECTOR(7 downto 0);  -- data write
		uc_rd:         in  STD_LOGIC;                     -- read strobe
		uc_wr:         in  STD_LOGIC;                     -- write strobe
		-- instruction ROM
		rom_adx:       out STD_LOGIC_VECTOR(15 downto 0); -- address
		rom_rdd:       in  STD_LOGIC_VECTOR(7 downto 0);  -- data
		-- user interface
		dout:          out STD_LOGIC_VECTOR(15 downto 0);
		din:           in  STD_LOGIC_VECTOR(15 downto 0)
		);
end xsvGlue;

--------------------------------------------------------------------------------
architecture BHV of xsvGlue is
	signal fifo:  STD_LOGIC;
	signal ctrl:  STD_LOGIC;
	signal dadx:  STD_LOGIC;
	signal leds:  STD_LOGIC;
	signal irom:  STD_LOGIC;
	signal doutii: STD_LOGIC_VECTOR(15 downto 0);
begin
	
	------------------------------------
	-- address decoding
	leds<= '1' when uc_addr(15 downto 12)= "1111" else '0'; -- 0xFxxx
	fifo<= '1' when uc_addr(15 downto 12)= "1110" else '0'; -- 0xExxx
	ctrl<= '1' when uc_addr(15 downto 12)= "1101" else '0'; -- 0xDxxx
	dadx<= '1' when uc_addr(15 downto 12)= "1100" else '0'; -- 0xCxxx
	irom<= '1' when uc_addr(15 downto 12)= "0000" else '0'; -- 0 to 4k
	
	------------------------------------
	-- EP #0 FIFO
	usb_adx   <= uc_addr(2 downto 0) when fifo= '1' else (others=> '-');
	usb_wrd   <= uc_wrd              when fifo= '1' else (others=> '-');
	uc_rdd    <= usb_rdd             when fifo= '1' else (others=> 'Z');
	usb_wren  <= not(uc_wr)          when fifo= '1' else '0';
	
	------------------------------------
	-- EP #0 control/ status
	usb_ctrl  <= uc_wrd              when ctrl= '1' else (others=> '-');
	uc_rdd    <= usb_stat            when ctrl= '1' else (others=> 'Z');
	usb_wrctrl<= not(uc_wr)          when ctrl= '1' else '0';
	
	------------------------------------
	-- device address
	usb_dadx  <= uc_wrd(6 downto 0)  when dadx= '1' else (others=> '-');
	usb_wradx <= not(uc_wr)          when dadx= '1' else '0';
	
	------------------------------------
	-- instruction ROM
	rom_adx   <= uc_addr             when irom= '1' else (others=> '-');
	uc_rdd    <= rom_rdd             when irom= '1' else (others=> 'Z');
	
	------------------------------------
	-- user registers
	process(clk, rst)
	begin
		if rst= '1' then
			doutii<= (others=> '0');
		elsif rising_edge(clk) then
			if leds= '1' and uc_wr= '0' then
				case uc_addr(1 downto 0) is
					when "00"=>   doutii(7  downto 0)<= uc_wrd;
					when "01"=>   doutii(15 downto 8)<= uc_wrd;
					when others=> null;
				end case;
			end if;
		end if;
	end process;
	dout<= doutii;
	uc_rdd  <= din(7  downto 0) when leds= '1' and uc_addr(1 downto 0)= "00" 
	else       din(15 downto 8) when leds= '1' and uc_addr(1 downto 0)= "01"
	else       (others=> 'Z');
	
	------------------------------------
	-- EP #1
	datain_blk: block
		signal txen1, txen: STD_LOGIC;             -- usb_txen shortened to 1 clk cycle
		signal bytecount:   INTEGER range 1 to 2;  -- byte counter
		begin
		
		process(clk, rst)
		begin
			if rst= '1' then
				txen1<= '0';
			elsif rising_edge(clk) then
				txen1<= usb_txen;
			end if;
		end process;
		txen<= '1' when usb_txen= '0' and txen1= '1' else '0';
		
		process(clk, rst)
		begin
			if rst= '1' then
				bytecount  <= 1;
				usb_datain1<= '0';
			elsif rising_edge(clk) then
				if usb_in_trac1= '0' then
					bytecount  <= 1;
					usb_datain1<= '1';
				elsif txen= '1' then
					if bytecount< 2 then
						bytecount<= bytecount + 1;
					else
						usb_datain1<= '0';
					end if;
				end if;
			end if;
		end process;
		
		with bytecount select usb_txd1<=
		din(7  downto 0) when 1,
		din(15 downto 8) when 2,
		"--------" when others;
	end block;
	
	toggle_blk: block
		signal in_trac:  STD_LOGIC;
		signal togglein: STD_LOGIC;
		begin
		process(clk, rst)
		begin
			if rst= '1' then
				in_trac <= '0';
				togglein<= '0';
			elsif rising_edge(clk) then
				if usb_in_trac1= '1' and in_trac= '0' then
					togglein<= not(togglein);
				end if;
				in_trac<= usb_in_trac1;
			end if;
		end process;
		usb_togglein1<= togglein;
	end block;
	-- nak, stall<= '0';
	
end BHV;

--------------------------------------------------------------------------------
-- end of file
